@@ -31,7 +31,13 @@ File::Copy::Recursive::dircopy( 't/lib', 't/tmp/TestApp/lib' );
 # remove TestApp's tests so Apache::Test doesn't try to run them
 rmtree 't/tmp/TestApp/t';
 
-$ENV{CATALYST_SERVER} = 'http://localhost:8529/fastcgi';
+$ENV{CATALYST_SERVER} = 'http://localhost:8529';
+
+if ( !-e 't/optional_apache-fastcgi.pl' ) {
+    die "ERROR: Please run test from the Catalyst-Runtime directory\n";
+}
+
+push @ARGV, glob( 't/live_*' );
 
 Apache::TestRun->new->run(@ARGV);
 
